@@ -31,10 +31,12 @@ use File::Basename 'basename';
 use base 'Exporter';
 
 $AUTHORITY = 'SUKRIA';
-$VERSION   = '1.1999_04';
+$VERSION   = '1.2000_01';
 @EXPORT    = qw(
   after
   any
+  apply_layout
+  apply_renderer
   before
   before_template
   cookies
@@ -69,6 +71,7 @@ $VERSION   = '1.1999_04';
   put
   r
   redirect
+  render_with_layout
   request
   send_file
   send_error
@@ -97,6 +100,8 @@ $VERSION   = '1.1999_04';
 
 sub after           { Dancer::Route::Registry->hook('after',           @_) }
 sub any             { Dancer::App->current->registry->any_add(@_) }
+sub apply_layout    { Dancer::Helpers::apply_layout(@_) }
+sub apply_renderer  { Dancer::Helpers::apply_renderer(@_) }
 sub before          { Dancer::Route::Registry->hook('before',          @_) }
 sub before_template { Dancer::Route::Registry->hook('before_template', @_) }
 sub captures        { Dancer::SharedData->request->params->{captures} }
@@ -135,6 +140,7 @@ sub options { Dancer::App->current->registry->universal_add('options', @_) }
 sub put     { Dancer::App->current->registry->universal_add('put',     @_) }
 sub r { carp "'r' is DEPRECATED use qr{} instead"; return {regexp => $_[0]} }
 sub redirect  { Dancer::Helpers::redirect(@_) }
+sub render_with_layout { Dancer::Helpers::render_with_layout(@_); }
 sub request   { Dancer::SharedData->request }
 sub send_file { Dancer::Helpers::send_file(@_) }
 sub set       { goto &setting }
@@ -367,6 +373,55 @@ Or even, a route handler that would match any HTTP methods:
         # code
     };
 
+=head2 apply_layout
+
+It is the second part of what C<template> does internally. Basically,
+C<template> is C<apply_renderer> + C<apply_layout>.
+
+C<apply_layout> allows a handler to provide plain HTML (or other content), but
+have it rendered within the layout still.
+
+For example:
+
+    get '/foo' => sub {
+        # Do something which generates HTML directly (maybe using
+        # HTML::Table::FromDatabase or something)
+        my $content = ...;
+        apply_layout $content;
+    };
+
+
+You can pass tokens to be used in the layout, and/or options to control the way
+the layout is rendered. For instance, to set a 'foo' token to 'bar', and use a
+custom layout:
+
+    apply_layout $content, { foo => 'bar' }, { layout => 'layoutname' };
+
+Remember, C<apply_layout> doesn't generates the content, it only applies the
+layout around it. So, C<$content> should be something semi final, like HTML. If
+you need to generate the content from a view, you should use C<template>, or
+C<apply_renderer>.
+
+=head2 apply_renderer
+
+It is the first part of what C<template> does internally. Basically,
+C<template> is C<apply_renderer> + C<apply_layout>.
+
+C<apply_renderer> generates content from a view, but doesn't apply the layout.
+
+For example:
+
+    get '/foo' => sub {
+        my $content = apply_renderer('home');
+        # $content now contains the rendered 'home' view
+        # .. do something with $content
+    };
+
+You can pass tokens to be used in the view. For instance, to set a 'foo' token
+to 'bar':
+
+    apply_renderer $content, { foo => 'bar'};
+
 =head2 before
 
 Defines a before filter:
@@ -675,6 +730,10 @@ function, e.g.
         return 'Welcome to the restricted section';
     };
 
+=head2 render_with_layout
+
+B<DEPRECATED> : this keyword is B<deprecated>. Please use C<apply_layout> instead.
+
 =head2 request
 
 Returns a L<Dancer::Request> object representing the current request.
@@ -802,7 +861,9 @@ underscores as a separator for blanks.
 
 =head2 template
 
-Tells the route handler to build a response with the current template engine:
+Basically, C<template> is C<apply_renderer> + C<apply_layout>.
+
+C<template> tells the route handler to build a response with the current template engine:
 
     get '/' => sub {
         ...
@@ -813,10 +874,11 @@ The first parameter should be a template available in the views directory, the
 second one (optional) is a hashref of tokens to interpolate, and the third
 (again optional) is a hashref of options.
 
-For example, to disable the layout for a specific request:
+For example, to set the 'foo' token to 'bar', but disable the layout for a
+specific request:
 
     get '/' => sub {
-        template 'index.tt', {}, { layout => undef };
+        template 'index.tt', { foo => bar }, { layout => undef };
     };
 
 
