@@ -18,8 +18,6 @@ use vars qw(@EXPORT);
 
 sub register($&);
 
-my @_reserved_keywords = @Dancer::EXPORT;
-
 my $_keywords = {};
 
 sub add_hook { Dancer::Route::Registry->hook(@_) }
@@ -43,8 +41,11 @@ sub register($&) {
     my ($keyword, $code) = @_;
     my $plugin_name = caller();
 
-    if (grep { $_ eq $keyword } @_reserved_keywords) {
-        croak "You can't use $keyword, this is a reserved keyword";
+    $keyword =~ /^[a-zA-Z_]+[a-zA-Z0-9_]*$/
+      or croak "You can't use '$keyword', it is an invalid name (it should match ^[a-zA-Z_]+[a-zA-Z0-9_]*$ )";
+
+    if (grep { $_ eq $keyword } map { s/^(?:\$|%|&|@|\*)//; $_ } (@Dancer::EXPORT, @Dancer::EXPORT_OK)) {
+        croak "You can't use '$keyword', this is a reserved keyword";
     }
     while (my ($plugin, $keywords) = each %$_keywords) {
         if (grep { $_->[0] eq $keyword } @$keywords) {
@@ -63,8 +64,9 @@ sub register_plugin {
     my @symbols = set_plugin_symbols($plugin);
     {
         no strict 'refs';
-        @{"${plugin}::ISA"} = ('Exporter', 'Dancer::Plugin');
-        @{"${plugin}::EXPORT"} = @symbols;
+        # tried to use unshift, but it yields an undef warning on $plugin (perl v5.12.1)
+        @{"${plugin}::ISA"} = ('Exporter', 'Dancer::Plugin', @{"${plugin}::ISA"});
+        push @{"${plugin}::EXPORT"}, @symbols;
     }
     return 1;
 }
